//=============================================================================
//  DWFire: Dynamic Lit Fire Emitter
//  A fire emitter that gives off a flickering light, can be triggered to go
//  'out' or 'light up'.
//
//  By Mark 'MarZer' Gillard
//=============================================================================
class DWFire extends DWEmitter
	hidecategories(Emitter)
	placeable;

#exec OBJ LOAD FILE="..\StaticMeshes\DWeather-smesh.usx"
#exec OBJ LOAD FILE="..\Textures\DWeather-Tex.utx"
	
var float CalculationInterval, Mod, DiffMod, InitRad, InitBright, Increment, SoundIncrement;
var() float MaxBrightness;
var() float MinBrightness;
var() color MaxColour;
var() color MinColour;
var color CurrentColour;
var() float Variance;
var() float RadialFluctuation;
var() enum EMode {FM_Burning,FM_Dead,} FireMode;
var bool bInitiallyBurning;
var enum ECurrent {FadingUp,Burning,FadingDown,Dead} Current;
var float CurrentTime;
var DWFireFearSpot PhearMeh;
var byte CurHue;
var byte CurSat;
var byte CurLum;

var class<Emitter> FireEffectClass;
var Emitter FireEffect;

replication
{
    reliable if ( Role == ROLE_Authority )
        Current, CurrentTime, bInitiallyBurning, FireMode, RadialFluctuation, Variance,
        CurrentColour, MinColour, MaxColour, MaxBrightness, MinBrightness,
        CalculationInterval, Mod, DiffMod, InitRad, InitBright, Increment, SoundIncrement, 
        FireSwitch, HurtMe, TriggeredFireSwitch, FireEffectClass, FireEffect;
}

simulated event PreBeginPlay()
{
	DiffMod = Variance * (MaxBrightness - MinBrightness);
	InitRad = LightRadius;
	InitBright = (MaxBrightness : MinBrightness);
	Increment = (CalculationInterval * (MaxBrightness - MinBrightness)) / 0.2;
	SoundIncrement = (CalculationInterval * 200) / 0.2;
	
	if (FireMode == FM_Burning)
	{
		bInitiallyBurning = true;
		SetTimer(CalculationInterval, true);
		Current = Burning;
		SoundVolume = 200;
		
		FireEffect = spawn(class'DWFireEmitter',self,,Location);
			FireEffect.SetBase(self);
		PhearMeh = spawn(class'DWFireFearSpot',,,Location);	
			PhearMeh.SetBase(self);
	}
	else
	{
		Current = Dead;
		bInitiallyBurning = false;
		LightBrightness = 0.0;
		SoundVolume = 0;
	}
}

simulated function HurtMe(int PainScale)
{	
	local Pawn Other;
	local SVehicle Car;
	
	foreach VisibleCollidingActors(class'Pawn', Other, 100, Location, true) 
	{
		if (Other != None)
		{
			Other.TakeDamage((1 * PainScale), None, Other.Location, vect(0,0,0), class'DamTypeFire');
		}
	}
	
	foreach VisibleCollidingActors(class'SVehicle', Car, 225, Location, true)
	{
		if (Car != None)
		{
			Car.TakeDamage((5 * PainScale), None, Car.Location, vect(0,0,0), class'DamTypeFireCAR');
		}
	}	
}

simulated function FireSwitch()
{
	CurrentTime = CurrentTime + CalculationInterval;
	
		if (Current == Burning)
		{
			Mod = RandRange(-1, 1) * DiffMod;
			LightRadius = InitRad + (RandRange(-1, 1) * RadialFluctuation);
			CurrentColour = RandomColour(MinColour, MaxColour);
			LightBrightness = InitBright + Mod;
			SoundVolume = 200;
			RGBToHLS(CurrentColour, CurHue, CurLum, CurSat);
			LightHue = CurHue;
			
			if (CurrentTime > 0.2)
			{
				HurtMe(2);
				CurrentTime=0.0;
			}
		}
		else if (Current == FadingUp)
		{
			if (LightBrightness < InitBright)
			{
				LightBrightness = Charge(Increment, LightBrightness, true) * (RandRange(0.9, 1.1));
				SoundVolume = Charge(SoundIncrement, SoundVolume, true) * (RandRange(0.9, 1.1));
				CurrentColour = RandomColour(MinColour, MaxColour);				
				RGBToHLS(CurrentColour, CurHue, CurLum, CurSat);
				LightHue = CurHue;
				
				if (CurrentTime > 0.2)
				{
					HurtMe(1);
		    		CurrentTime=0.0;
				}
			}
			else
			{
				Current = Burning;
			}
		}
		else if (Current == FadingDown)
		{
			if (LightBrightness > 0)
			{
				LightBrightness = Charge(Increment, LightBrightness, false) * (RandRange(0.9, 1.1));
				SoundVolume = Charge(SoundIncrement, SoundVolume, false) * (RandRange(0.9, 1.1));
				CurrentColour = RandomColour(MinColour, MaxColour);				
				RGBToHLS(CurrentColour, CurHue, CurLum, CurSat);
				LightHue = CurHue;
			
				if (CurrentTime > 0.2)
				{
					HurtMe(1);
					CurrentTime=0.0;
				}
			}
			else
			{
			Current = Dead;
			HurtMe(0);
			SetTimer(0.0, false);
			}
		}
}	

simulated function Timer() 
{
	FireSwitch();
}	

simulated function TriggeredFireSwitch()
{
	if (FireMode == FM_Burning)
     	{         
	     	Current = FadingDown;
	     	FireMode = FM_Dead;

			if ( FireEffect != None )
				FireEffect.Kill();
			
			if (PhearMeh != None)
				PhearMeh.Destroy();
		}
    else
     	{         
			SetTimer(CalculationInterval, true);
			FireMode = FM_Burning;
	     	Current = FadingUp;

			if (!bInitiallyBurning)
			{
				FireEffect = spawn(class'DWFireEmitter',self,,Location);
					FireEffect.SetBase(self);
				bInitiallyBurning = true;	
			}
			
			if (FireEffect == None)
				FireEffect = spawn(class'DWFireEmitter',self,,Location);
					FireEffect.SetBase(self);
							
			if (PhearMeh == None)
				PhearMeh = spawn(class'DWFireFearSpot',,,Location);	
					PhearMeh.SetBase(self);
		}
}	

simulated function Trigger( actor Other, pawn EventInstigator )
{
	 TriggeredFireSwitch();    
} 
	
defaultproperties
{
    Texture=Texture'DWeather-Tex.Icons.Fireicon'
   	CalculationInterval=0.05
   	Mod=0.0
   	DiffMod=0.0
   	Variance=1.2
   	RadialFluctuation=1.0
    LightType=LT_Steady
	LightEffect=LE_None
	bDynamicLight=true
	LightCone=128
	LightRadius=8
	LightPeriod=32
	bAlwaysRelevant=true
    bStatic=false
	RemoteRole=ROLE_SimulatedProxy
    MaxColour=(R=151,G=75,B=0)
   	MinColour=(R=253,G=214,B=159)
   	MaxBrightness=90
   	MinBrightness=80
   	LightSaturation=127
	LightBrightness=85
    LightHue=21
    SoundVolume=200
    SoundPitch=50
    SoundRadius=200.000000
    AmbientSound=Sound'GeneralAmbience.firefx3'
    FireMode=FM_Burning
	CurrentTime=0.0
	bCollideActors=true
	bNetTemporary=false
	bUpdateSimulatedPosition=true
	bHidden=true
}
