//=============================================================================
//  DWMeteor
//  A Meteor spawner
//  By Mark 'MarZer' Gillard
//=============================================================================
class DWMeteor extends Actor
	placeable;
	
#exec OBJ LOAD FILE="..\StaticMeshes\DWeather-smesh.usx"
#exec OBJ LOAD FILE="..\Textures\DWeather-Tex.utx"

var() float SpawnIntervalMin;
var() float SpawnIntervalMax;
var float 	ActualSpawnInterval;
var() float RoamingSpeed;
var() bool 	Roaming;
var() bool 	RandomReflectionAngle;
var() bool 	Enabled;
var vector 	RotVec;
var rotator newRot;
var rotator Initial;
var() enum EOnTrigger {OT_SpawnMeteor,OT_ToggleEnabled,} OnTrigger;

replication
{
    reliable if ( Role == ROLE_Authority )
    ChangeHeading, SpawnMeteor;
}
	
simulated function PostBeginPlay()
{
	if(Roaming)
	{
		Initial.Pitch = 0;
		Initial.Yaw = Rotation.Yaw;
		Initial.Roll = 0;
		SetRotation(Initial);
		
		RotVec = normal(vector(Rotation)) * RoamingSpeed;
		RotVec.Z = 0;
		Velocity = RotVec;
	}
	
	ActualSpawnInterval = RandRange(SpawnIntervalMin, SpawnIntervalMax);
	
	if(Enabled)
		SetTimer(ActualSpawnInterval, false);
}

simulated function ChangeHeading(bool Random)
{
	newRot = Rotation;
	
    if (Random)
    {
		newRot.Yaw = (newRot.Yaw + 32768) * RandRange(0.55, 1.45);
	}
	else
	{
		newRot.Yaw = newRot.Yaw + 32768;
	}		
    
	SetRotation(newRot);
	Velocity = normal(vector(newRot)) * RoamingSpeed;
}

simulated function SpawnMeteor()
{
	local rotator PitchRotator;
	
	if(Roaming)
	{
		PitchRotator.Pitch = -16384 * RandRange(0.6, 1.4);
		PitchRotator.Yaw = RandRange(-32768, 32768);
	}
	else
	{
		PitchRotator = Rotation;
	}
	
	spawn(class'DWMeteorProjectile',,,Location,PitchRotator);

	ChangeHeading(RandomReflectionAngle);
}

simulated function HitWall( vector HitNormal, actor Wall )
{
	ChangeHeading(RandomReflectionAngle);
}

simulated function Timer()
{
	SpawnMeteor();

	ActualSpawnInterval = RandRange(SpawnIntervalMin, SpawnIntervalMax);
	SetTimer(ActualSpawnInterval, false);
}
	

simulated function Trigger( actor Other, Pawn EventInstigator )
{
	if(OnTrigger == OT_SpawnMeteor)
		SpawnMeteor();
	else
	{
		if(Enabled)
		{	
			Enabled = false;
			SetTimer(0.0, false);
		}
		else
		{
			Enabled = true;
			SetTimer(ActualSpawnInterval, false);
		}
	}
}
	
defaultproperties
{
   	Texture=Texture'DWeather-Tex.Icons.meteoricon'

    bDirectional=true

    bAlwaysRelevant=true
    bNoDelete=true
	bStatic=false
 	bMovable=true
	RemoteRole=ROLE_SimulatedProxy
	bReplicateMovement=true
	bNetTemporary=false
	bUpdateSimulatedPosition=true
 	 	
	Physics=PHYS_Projectile
	bCollideActors=true
	bCollideWorld=true
	CollisionHeight=3.0
    CollisionRadius=3.0
    bUseCylinderCollision=true
    RandomReflectionAngle=true
    
    bHidden=true
    
    OnTrigger=OT_ToggleEnabled
    Enabled=true
   	RoamingSpeed=500
   	Roaming=true
   	SpawnIntervalMax=10
   	SpawnIntervalMin=4
}