//=============================================================================
// DWMeteorFlyingRock.
//=============================================================================
class DWMeteorFlyingRock extends Projectile;

var Emitter FlyingEffect;
var class<Emitter> FlyingEffectClass;

var byte Bounces;
var float DamageAtten;
var sound ImpactSounds[6];

replication
{
    reliable if (bNetInitial && Role == ROLE_Authority)
        Bounces;
}

simulated function Destroyed()
{
	if ( FlyingEffect != None )
		FlyingEffect.Kill();
	
	Super.Destroyed();
}

simulated function PostBeginPlay()
{
    local float r;

    if ( Level.NetMode != NM_DedicatedServer )
    {
        if ( !PhysicsVolume.bWaterVolume )
        {
   			FlyingEffect = spawn(FlyingEffectClass, self);
	
			if (FlyingEffect != None)
				FlyingEffect.SetBase(self); 
        }
            
    }

    Velocity = Vector(Rotation) * (Speed);
    
    if (PhysicsVolume.bWaterVolume)
        Velocity *= 0.65;

    r = FRand();
    if (r > 0.75)
        Bounces = 2;
    else if (r > 0.25)
        Bounces = 1;
    else
        Bounces = 0;

    SetRotation(RotRand());
	RandSpin(100000);
    
    Super.PostBeginPlay();
}

simulated function ProcessTouch (Actor Other, vector HitLocation)
{
    if ( (FlakChunk(Other) == None) && ((Physics == PHYS_Falling) || (Other != Instigator)) )
    {
        speed = VSize(Velocity);
        if ( speed > 200 )
        {
            if ( Role == ROLE_Authority )
			{
				if ( Instigator == None || Instigator.Controller == None )
					Other.SetDelayedDamageInstigatorController( InstigatorController );

                Other.TakeDamage( Max(5, Damage - DamageAtten*FMax(0,(default.LifeSpan - LifeSpan - 1))), Instigator, HitLocation,
                    (MomentumTransfer * Velocity/speed), MyDamageType );
			}
        }
        Destroy();
    }
}

simulated function Landed( Vector HitNormal )
{
    SetPhysics(PHYS_None);
    LifeSpan = 1.0;
}

simulated function HitWall( vector HitNormal, actor Wall )
{
    if ( !Wall.bStatic && !Wall.bWorldGeometry 
		&& ((Mover(Wall) == None) || Mover(Wall).bDamageTriggered) )
    {
        if ( Level.NetMode != NM_Client )
		{
			if ( Instigator == None || Instigator.Controller == None )
				Wall.SetDelayedDamageInstigatorController( InstigatorController );
            Wall.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
		}
        Destroy();
        return;
    }

    SetPhysics(PHYS_Falling);
	if (Bounces > 0)
    {
		if ( !Level.bDropDetail && (FRand() < 0.4) )
			Playsound(ImpactSounds[Rand(6)]);

        Velocity = 0.65 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
        Bounces = Bounces - 1;
        return;
    }
	bBounce = false;
}

simulated function PhysicsVolumeChange( PhysicsVolume Volume )
{
    if (Volume.bWaterVolume)
    {
        Velocity *= 0.65;
        FlyingEffect.Kill();        
    }
}

defaultproperties
{
    Style=STY_Alpha
    ScaleGlow=1.0
    DrawType=DT_StaticMesh
    StaticMesh=StaticMesh'DWeather-smesh.Meteor.rock-chunk'
    MyDamageType=class'DamTypeMeteorFlyingRock'
	FluidSurfaceShootStrengthMod=1.f
    speed=2000.000000
    MaxSpeed=2000.000000
    Damage=50
    DamageAtten=3.0
    MomentumTransfer=3000
    Bounces=2
    Lifespan=10
    RotationRate=(Pitch=800,Yaw=1600,Roll=1000)
    DesiredRotation=(Pitch=400,Yaw=1200,Roll=600)
    bBounce=true
    NetPriority=2.500000
    AmbientGlow=254
    DrawScale=1.0
    CullDistance=+3000.0
	ImpactSounds(0)=sound'XEffects.Impact4Snd'
	ImpactSounds(1)=sound'XEffects.Impact6Snd'
	ImpactSounds(2)=sound'XEffects.Impact7Snd'
	ImpactSounds(3)=sound'XEffects.Impact3'
	ImpactSounds(4)=sound'XEffects.Impact1'
	ImpactSounds(5)=sound'XEffects.Impact2'
    Physics=PHYS_Falling	
    FlyingEffectClass=class'DWMeteorFlyingRockTrail'  
}
