//-----------------------------------------------------------
//
//-----------------------------------------------------------
class DWMeteorProjectile extends Projectile;

#exec OBJ LOAD FILE="..\Sounds\VMVehicleSounds-S.uax"

var class<Emitter> FlyingEffectClass;
var Emitter FlyingEffect;

var bool bHitWater;
var vector Dir;

simulated function Destroyed()
{
	if ( FlyingEffect != None )
		FlyingEffect.Kill();
		
	Super.Destroyed();
}

simulated function PostBeginPlay()
{
	if ( Level.NetMode != NM_DedicatedServer)
	{
		FlyingEffect = spawn(FlyingEffectClass, self);
		
		if (FlyingEffect != None)
			FlyingEffect.SetBase(self);
	}
	
	Dir = vector(Rotation);
	Velocity = speed * Dir;
	
	if (PhysicsVolume.bWaterVolume)
	{
		bHitWater = True;
		Velocity=0.6*Velocity;
	}
    if ( Level.bDropDetail )
	{
		bDynamicLight = false;
		LightType = LT_None;
	}
	Super.PostBeginPlay();
	RandSpin(200000);
}

simulated function Landed( vector HitNormal )
{
	Explode(Location,HitNormal);
}

simulated function ProcessTouch (Actor Other, Vector HitLocation)
{
	if ( (Other != instigator) && (!Other.IsA('Projectile') || Other.bProjTarget) )
		Explode(HitLocation,Vect(0,0,1));
}

function BlowUp(vector HitLocation)
{
	HurtRadius(Damage, DamageRadius, MyDamageType, MomentumTransfer, HitLocation );
	MakeNoise(1.0);
}

simulated function Explode(vector HitLocation, vector HitNormal)
{
	PlaySound(sound'WeaponSounds.BExplosion3',,7.5*TransientSoundVolume);
    if ( EffectIsRelevant(Location,false) )
    {
    	spawn(class'DWMeteorExplosion',,,HitLocation + HitNormal*16,rotator(HitNormal));
		if ( (ExplosionDecal != None) && (Level.NetMode != NM_DedicatedServer) )
			spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
    }
    BlowUp(HitLocation);
    SpawnChunks(HitNormal);
	Destroy();
}

simulated function SpawnChunks(vector HitNormal)
{
	local vector Start;
    local rotator rot;
    local int i;
    local DWMeteorFlyingRock NewChunk;
	
    Start = Location + 10 * HitNormal;
    
	if ( Role == ROLE_Authority )
	{
		for (i=0; i<23; i++)
		{
			rot = Rotation;
			rot.yaw += FRand()*32000-16000;
			rot.pitch += FRand()*32000-16000;
			rot.roll += FRand()*32000-16000;
			NewChunk = spawn( class 'DWMeteorFlyingRock',, '', Start, rot);
		}
	}
}


simulated function PhysicsVolumeChange( PhysicsVolume Volume )
{
	if (Volume.bWaterVolume)
    {
        Velocity *= 0.65;
        FlyingEffect.Kill();        
    }
}
defaultproperties
{
    Speed=6000.0
    MaxSpeed=6000.0
    MomentumTransfer=125000
    Damage=1000.0
    DamageRadius=750.0
    AmbientSound=sound'VMVehicleSounds-S.HoverTank.IncomingShell'
    SoundVolume=255
    SoundRadius=2500   
    TransientSoundVolume=1.0
    TransientSoundRadius=1000
    bFullVolume=True
    MyDamageType=class'DamTypeMeteor'
  	ExplosionDecal=class'DWMeteorScorch'
    RemoteRole=ROLE_SimulatedProxy
    LifeSpan=6
    DrawType=DT_StaticMesh
    StaticMesh=StaticMesh'DWeather-smesh.Meteor.Meteor-ball'
    DrawScale=3   
    AmbientGlow=140
    bUnlit=true
    bBounce=false
    bFixedRotationDir=True
    RotationRate=(Pitch=800,Yaw=1600,Roll=1000)
    DesiredRotation=(Pitch=400,Yaw=1200,Roll=600)
    ForceType=FT_Constant
    ForceScale=5.0
    ForceRadius=100.0
    bCollideWorld=true
    bUseCollisionStaticMesh=true
    FluidSurfaceShootStrengthMod=40.0
    CollisionHeight=1.0
    CollisionRadius=1.0
  	FlyingEffectClass=class'DWMeteorFlames'
  	Physics=PHYS_Projectile
}